#include "NiProvider.h"
#include <iostream>

using namespace std;
using namespace thomsonreuters::ema::access;
using namespace thomsonreuters::ema::rdm;
using namespace thomsonreuters::tutorials::ema::nip;

//--------------------------------------------------------------------------------

NiProvider::NiProvider()
	:_theOnlyOneItem("SHARE-0", 14400)
{
	cout << "  Provider created" << endl;
}

//--------------------------------------------------------------------------------

NiProvider::~NiProvider()
{
	disconnect();
}

//--------------------------------------------------------------------------------

void NiProvider::setServiceName(const std::string & serviceName)
{
	_serviceName = serviceName;
}

//--------------------------------------------------------------------------------

const std::string & NiProvider::getServiceName() const
{
	return _serviceName;
}

//--------------------------------------------------------------------------------

void NiProvider::connectAs(const std::string & userName)
{

	// Exit the method if already connected
	if (_provider != 0)
		return;

	cout << "  Connecting Provider to the ADH as " << userName << endl;

	// Create a Non Interactive Provider using harcoded parameters
	_provider = new OmmProvider(
						OmmNiProviderConfig()
						.username(userName.c_str()));

	// Register the _connectionStateListener to receive login messages from EMA
	_provider->registerClient(
						ReqMsg()
						.domainType(MMT_LOGIN)
						.name(userName.c_str()),
						_connectionStateListener,
						(void*)0);
}

//--------------------------------------------------------------------------------

void NiProvider::disconnect()
{
	// Exit the method if already disconnected
	if (_provider == 0)
		return;

	cout << "  Disconnecting..." << endl;

	// Disconnect from the infrastructure and automatically unregister the 
	// _connectionStateListener we registered in the connect() method to 
	// receive login messages from EMA
	delete _provider;
	_provider = 0;
}

//--------------------------------------------------------------------------------

bool NiProvider::isConnected() const 
{
	return _connectionStateListener.isConnected();
}

//--------------------------------------------------------------------------------

void NiProvider::refresh(const std::string & itemName)
{
	if (!isConnected())
	{
		cout << "  ERROR: Can't refresh " << itemName << ". The provider is not connected." << endl;
		return;
	}

	if (itemName != _theOnlyOneItem.getName())
	{
		cout << "  ERROR: Can't refresh " << itemName << ". Unknown item name." << endl;
		return;
	}

	Item & item = _theOnlyOneItem;

	cout << "  Refreshing " << item.getName() << endl;

	_provider->submit(
					RefreshMsg()
						.serviceName(getServiceName().c_str())
						.name(item.getName().c_str())
						.domainType(MMT_MARKET_PRICE)
						.state(
							OmmState::OpenEnum, 
							OmmState::OkEnum, 
							OmmState::NoneEnum, 
							"UnSolicited Refresh Completed")
						.payload(FieldList()
							.addAscii(
								DSPLY_NAME, 
								item.getDisplayName().c_str())
							.addReal(
								BID, 
								item.getBidPrice(), 
								OmmReal::ExponentNeg2Enum)
							.addReal(
								ASK, 
								item.getAskPrice(), 
								OmmReal::ExponentNeg2Enum)
							.addReal(
								OPEN_PRC, 
								item.getOpenPrice(), 
								OmmReal::ExponentNeg2Enum)
							.addReal(
								HST_CLOSE, 
								item.getClosePrice(), 
								OmmReal::ExponentNeg2Enum)
							.complete())
						.complete(), 
					item.getHandle());
}

//--------------------------------------------------------------------------------

void NiProvider::update(const std::string & itemName)
{
	if (!isConnected())
	{
		cout << "  ERROR: Can't update " << itemName << ". The provider is not connected." << endl;
		return;
	}

	if (itemName != _theOnlyOneItem.getName())
	{
		cout << "  ERROR: Can't update " << itemName << ". Unknown item name." << endl;
		return;
	}

	Item & item = _theOnlyOneItem;

	cout << "  Updating " << item.getName() << endl;

	item.generateNextTick();

	_provider->submit(
					UpdateMsg()
						.serviceName(getServiceName().c_str())
						.name(item.getName().c_str())
						.payload(FieldList()
							.addReal(
								BID,
								item.getBidPrice(),
								OmmReal::ExponentNeg2Enum)
							.addReal(
								ASK,
								item.getAskPrice(),
								OmmReal::ExponentNeg2Enum)
							.complete()),
					item.getHandle());
}

//--------------------------------------------------------------------------------

void NiProvider::sendOkStatusFor(const std::string & itemName)
{
	if (!isConnected())
	{
		cout << "  ERROR: Can't send a status for " << itemName << ". The provider is not connected." << endl;
		return;
	}

	if (itemName != _theOnlyOneItem.getName())
	{
		cout << "  ERROR: Can't send a status for " << itemName << ". Unknown item name." << endl;
		return;
	}

	Item & item = _theOnlyOneItem;

	cout << "  Sending an Ok status for " << item.getName() << endl;

	_provider->submit( 
					StatusMsg()
						.state(
							OmmState::OpenEnum,	// Stream state
							OmmState::OkEnum,	// Data state
							OmmState::NoneEnum, // Data status
							"All is good" ),	// Explanatory text
					item.getHandle());

}

//--------------------------------------------------------------------------------

void NiProvider::sendSuspectStatusFor(const std::string & itemName)
{
	if (!isConnected())
	{
		cout << "  ERROR: Can't send a status for " << itemName << ". The provider is not connected." << endl;
		return;
	}

	if (itemName != _theOnlyOneItem.getName())
	{
		cout << "  ERROR: Can't send a status for " << itemName << ". Unknown item name." << endl;
		return;
	}

	Item & item = _theOnlyOneItem;

	cout << "  Sending a Stale status for " << item.getName() << endl;

	_provider->submit(
					StatusMsg()
					.state(
						OmmState::OpenEnum,				// Stream state
						OmmState::SuspectEnum,			// Data state
						OmmState::ErrorEnum,			// Data status
						"Houston, we have a problem!"),	// Explanatory text
					item.getHandle());
}

//--------------------------------------------------------------------------------
