#include "NiProvider.h"
#include <iostream>
#include <sstream>

#define REUSE_EMA_OBJECTS

using namespace std;
using namespace thomsonreuters::ema::access;
using namespace thomsonreuters::ema::rdm;
using namespace thomsonreuters::tutorials::ema::nip;

//--------------------------------------------------------------------------------

NiProvider::NiProvider()
{
	cout << "  Provider created" << endl;
}

//--------------------------------------------------------------------------------

NiProvider::~NiProvider()
{
	disconnect();
}

//--------------------------------------------------------------------------------

void NiProvider::setServiceName(const std::string & serviceName)
{
	_serviceName = serviceName;
}

//--------------------------------------------------------------------------------

const std::string & NiProvider::getServiceName() const
{
	return _serviceName;
}

//--------------------------------------------------------------------------------

void NiProvider::connectAs(const std::string & userName)
{

	// Exit the method if already connected
	if (_provider != 0)
		return;

	cout << "  Connecting Provider to the ADH as " << userName << endl;

	// Create a Non Interactive Provider using harcoded parameters
	_provider = new OmmProvider(
						OmmNiProviderConfig()
						.username(userName.c_str()));

	// Register the _connectionStateListener to receive login messages from EMA
	_provider->registerClient(
						ReqMsg()
						.domainType(MMT_LOGIN)
						.name(userName.c_str()),
						_connectionStateListener,
						(void*)0);
}

//--------------------------------------------------------------------------------

void NiProvider::disconnect()
{
	// Exit the method if already disconnected
	if (_provider == 0)
		return;

	cout << "  Disconnecting..." << endl;

	// Disconnect from the infrastructure and automatically unregister the 
	// _connectionStateListener we registered in the connect() method to 
	// receive login messages from EMA
	delete _provider;
	_provider = 0;
}

//--------------------------------------------------------------------------------

bool NiProvider::isConnected() const 
{
	return _connectionStateListener.isConnected();
}

//--------------------------------------------------------------------------------

void NiProvider::createItems(const std::string & itemNamePrefix, unsigned int itemCount)
{
	// Clears the items collection
	_items.clear();

	for (unsigned int itemId = 0; itemId < itemCount; ++itemId)
	{
		// Generates the item name
		std::ostringstream oss;
		oss << itemNamePrefix << itemId;
		std::string itemName = oss.str();

		// Compute a base price for this item 
		int itemBasePrice = BASE_PRICE + itemId * 100;

		// Creates the item and preserves it in the collection
		_items[itemName] = Item(itemName, itemBasePrice);
	}
}

//--------------------------------------------------------------------------------

void NiProvider::refresh(const std::string & itemName)
{
	if (!isConnected())
	{
		cout << "  ERROR: Can't refresh " << itemName << ". The provider is not connected." << endl;
		return;
	}

	if(!itemExists(itemName))
	{
		cout << "  ERROR: Can't refresh " << itemName << ". Unknown item name." << endl;
		return;
	}

	Item & item = _items[itemName];

	cout << "  Refreshing " << item.getName() << endl;

	refresh(item);
}

//--------------------------------------------------------------------------------

void NiProvider::refresh(thomsonreuters::tutorials::ema::nip::Item & item)
{
	_provider->submit(
#if defined REUSE_EMA_OBJECTS
				_refreshMessage.clear()
#else
				RefreshMsg()
#endif
					.serviceName(getServiceName().c_str())
					.name(item.getName().c_str())
					.domainType(MMT_MARKET_PRICE)
					.state(
						OmmState::OpenEnum,
						OmmState::OkEnum,
						OmmState::NoneEnum,
						"UnSolicited Refresh Completed")
					.payload(
#if defined REUSE_EMA_OBJECTS
						_fieldList.clear()
#else
						FieldList()
#endif
							.addAscii(
								DSPLY_NAME,
								item.getDisplayName().c_str())
							.addReal(
								BID,
								item.getBidPrice(),
								OmmReal::ExponentNeg2Enum)
							.addReal(
								ASK,
								item.getAskPrice(),
								OmmReal::ExponentNeg2Enum)
							.addReal(
								OPEN_PRC,
								item.getOpenPrice(),
								OmmReal::ExponentNeg2Enum)
							.addReal(
								HST_CLOSE,
								item.getClosePrice(),
								OmmReal::ExponentNeg2Enum)
							.complete())
						.complete(),
				item.getHandle());
}

//--------------------------------------------------------------------------------

void NiProvider::refreshAll()
{
	cout << "  Refreshing all items" << endl;

	for (ItemsByName::iterator it=_items.begin(); it!=_items.end(); ++it)
	{
		Item & item = it->second;
		refresh(item);
	}
}

//--------------------------------------------------------------------------------

void NiProvider::update(const std::string & itemName)
{
	if (!isConnected())
	{
		cout << "  ERROR: Can't update " << itemName << ". The provider is not connected." << endl;
		return;
	}

	if (!itemExists(itemName))
	{
		cout << "  ERROR: Can't update " << itemName << ". Unknown item name." << endl;
		return;
	}

	Item & item = _items[itemName];

	cout << "  Updating " << item.getName() << endl;

	update(item);
}

//--------------------------------------------------------------------------------

void NiProvider::update(thomsonreuters::tutorials::ema::nip::Item & item)
{
	item.generateNextTick();

	_provider->submit(
#if defined REUSE_EMA_OBJECTS
				_updateMessage.clear()
#else
				UpdateMsg()
#endif
					.serviceName(getServiceName().c_str())
					.name(item.getName().c_str())
					.payload(
#if defined REUSE_EMA_OBJECTS
						_fieldList.clear()
#else
						FieldList()
#endif
							.addReal(
								BID,
								item.getBidPrice(),
								OmmReal::ExponentNeg2Enum)
							.addReal(
								ASK,
								item.getAskPrice(),
								OmmReal::ExponentNeg2Enum)
							.complete()),
				item.getHandle());
}

//--------------------------------------------------------------------------------

void NiProvider::updateAll()
{
	cout << "  Updating all items" << endl;

	for (ItemsByName::iterator it = _items.begin(); it != _items.end(); ++it)
	{
		Item & item = it->second;
		update(item);
	}
}

//--------------------------------------------------------------------------------

void NiProvider::sendOkStatusFor(const std::string & itemName)
{
	if (!isConnected())
	{
		cout << "  ERROR: Can't send a status for " << itemName << ". The provider is not connected." << endl;
		return;
	}

	if (!itemExists(itemName))
	{
		cout << "  ERROR: Can't send a status for " << itemName << ". Unknown item name." << endl;
		return;
	}

	Item & item = _items[itemName];

	cout << "  Sending an Ok status for " << item.getName() << endl;

	_provider->submit( 
					StatusMsg()
						.state(
							OmmState::OpenEnum, 
							OmmState::OkEnum, 
							OmmState::NoneEnum, 
							"All is good" ), 
					item.getHandle());

}

//--------------------------------------------------------------------------------

void NiProvider::sendSuspectStatusFor(const std::string & itemName)
{
	if (!isConnected())
	{
		cout << "  ERROR: Can't send a status for " << itemName << ". The provider is not connected." << endl;
		return;
	}

	if (!itemExists(itemName))
	{
		cout << "  ERROR: Can't send a status for " << itemName << ". Unknown item name." << endl;
		return;
	}

	Item & item = _items[itemName];

	cout << "  Sending a Stale status for " << item.getName() << endl;

	_provider->submit(
		StatusMsg()
		.state(
			OmmState::OpenEnum,
			OmmState::SuspectEnum,
			OmmState::ErrorEnum,
			"Houston, we have a problem!"),
		item.getHandle());
}

//--------------------------------------------------------------------------------

bool NiProvider::itemExists(const std::string & itemName) const
{
	if (_items.find(itemName) == _items.end())
		return false;
	
	return true;
}

//--------------------------------------------------------------------------------
