// *|-----------------------------------------------------------------------------
// *|            This source code is provided under the Apache 2.0 license      --
// *|  and is provided AS IS with no warranty or guarantee of fit for purpose.  --
// *|                See the project's LICENSE.md for details.                  --
// *|           Copyright Thomson Reuters 2016. All rights reserved.            --
// *|-----------------------------------------------------------------------------

// *|-----------------------------------------------------------------------------
// *|                   Non Interactive Provider EMA Tutorial                   --
// *|                                                                           --
// *|                Tutorial 9 - Publishing multiple data items                --
// *|                                                                           --
// *| This tutorial is the 9th part of a series of tutorials with the goal      --
// *| of creating a Non Iteractive Provider using the C++ edition of EMA, the   --
// *| Elektron Message API. See the README.md for more details.                 --
// *|-----------------------------------------------------------------------------

#include <iostream>

#ifdef WIN32
#include <windows.h>
#else
#include <sys/time.h>
#endif
#include <sys/timeb.h>

#include "NiProvider.h"

//--------------------------------------------------------------------------------

using namespace std;
using namespace thomsonreuters::tutorials::ema::nip;

//--------------------------------------------------------------------------------
// This function blocks the current thread by sleeping the number of seconds
// indicated by the "seconds" argument. It exits when the time is elapsed.
void waitFor(int millisecs);

//--------------------------------------------------------------------------------
// This function returns the current time expressed in millsieconds.
unsigned long long getCurrentTime();

//--------------------------------------------------------------------------------
// This function prints performance statistics.
void printStatistics(unsigned int itemsCount, unsigned long long startTime, unsigned long long endTime);

//--------------------------------------------------------------------------------
// This function implements the main workflow of the application. Actual Elektron
// Message API calls are delegated to the NiProvider class.
int main(int argc, char* argv[])
{
	try
	{
		cout << "-------------------------------------------------------------------------------" << endl;
		cout << "|                    Non Interactive Provider EMA Tutorial                    |" << endl;
		cout << "|                                                                             |" << endl;
		cout << "|                 Tutorial 9 - Publishing multiple data items                 |" << endl;
		cout << "-------------------------------------------------------------------------------" << endl;
		
		NiProvider provider;
		unsigned int itemsCount = 10000;
		unsigned long long startTime = 0;
		unsigned long long endTime = 0;

		provider.setServiceName("TEST_NI_PUB");
		provider.createItems("SHARE-", itemsCount);
		provider.connectAs("YOUR_PROVIDER_USER_NAME");
		waitFor(5);

		startTime = getCurrentTime();
		provider.refreshAll();
		endTime = getCurrentTime();
		printStatistics(itemsCount, startTime, endTime);
	
		waitFor(1);

		for (int i = 0; i < 30; ++i)
		{
			startTime = getCurrentTime();
			provider.updateAll();
			endTime = getCurrentTime();
			printStatistics(itemsCount, startTime, endTime);

			waitFor(1);
		}
		
		provider.disconnect();
	}
	catch (const thomsonreuters::ema::access::OmmException& excp)
	{
		cout << excp << endl;
	}

	cout << "  Exiting the application" << endl;
	return 0;
}

//--------------------------------------------------------------------------------

void waitFor(int seconds)
{
	cout << "  Waiting for " << seconds << " seconds..." << endl;

#if defined WIN32
	::Sleep((DWORD)(seconds * 1000));
#else
	struct timespec sleeptime;
	sleeptime.tv_sec = seconds;
	sleeptime.tv_nsec = 0;
	nanosleep(&sleeptime, 0);
#endif
}

//--------------------------------------------------------------------------------

unsigned long long getCurrentTime()
{
	unsigned long long msec = 0;
	struct	_timeb	_time;
	_ftime_s(&_time);
	msec = _time.time * 1000 + _time.millitm;
	return msec;
}

//--------------------------------------------------------------------------------

void printStatistics(unsigned int itemsCount, unsigned long long startTime, unsigned long long endTime)
{
	float timeSpent = (float)(endTime - startTime) / (float)1000;

	cout << "\tMessage count: " << itemsCount << endl
		 << "\tTotal time   : " << timeSpent << " sec" << endl
		 << "\tMessage rate : " << (float)itemsCount / timeSpent << " msg/sec" << endl;
}

//--------------------------------------------------------------------------------